@@ -28,7 +28,7 @@ _setflags () {
     shift;shift
     eval _tval=\$$_tvar
     dosubst _tval '^ *' ''
-    setdata ${_MKCONFIG_PREFIX} ${_tenm} "$_tval"
+    setdata ${_tenm} "$_tval"
   done
 }
 
@@ -48,15 +48,15 @@ _dogetconf () {
   if [ z${xgetconf} != z ]
   then
       puts "using flags from getconf" >&9
-      lfccflags="`${xgetconf} LFS_CFLAGS 2>/dev/null`"
+      lfccflags=`${xgetconf} LFS_CFLAGS 2>/dev/null`
       if [ "$lfccflags" = undefined ]; then
         lfccflags=
       fi
-      lfldflags="`${xgetconf} LFS_LDFLAGS 2>/dev/null`"
+      lfldflags=`${xgetconf} LFS_LDFLAGS 2>/dev/null`
       if [ "$lfldflags" = undefined ]; then
         lfldflags=
       fi
-      lflibs="`${xgetconf} LFS_LIBS 2>/dev/null`"
+      lflibs=`${xgetconf} LFS_LIBS 2>/dev/null`
       if [ "$lflibs" = undefined ]; then
         lflibs=
       fi
@@ -69,11 +69,10 @@ check_32bitflags () {
 
   printlabel _MKCONFIG_32BIT_FLAGS "32 bit flags"
   printyesno_val _MKCONFIG_32BIT_FLAGS "${_MKCONFIG_32BIT_FLAGS}"
-  setdata ${_MKCONFIG_PREFIX} _MKCONFIG_32BIT_FLAGS "${_MKCONFIG_32BIT_FLAGS}"
+  setdata _MKCONFIG_32BIT_FLAGS "${_MKCONFIG_32BIT_FLAGS}"
 }
 
 check_cc () {
-  _read_option CC cc
   CC=${CC:-${oval}}
 
   printlabel CC "C compiler"
@@ -98,9 +97,9 @@ check_cc () {
   puts "cc:${CC}" >&9
 
   printyesno_val CC "${CC}"
-  setdata ${_MKCONFIG_PREFIX} CC "${CC}"
+  setdata CC "${CC}"
   if [ ${_MKCONFIG_32BIT_FLAGS} = F ]; then
-    setdata ${_MKCONFIG_PREFIX} _MKCONFIG_32BIT_FLAGS "${_MKCONFIG_32BIT_FLAGS}"
+    setdata _MKCONFIG_32BIT_FLAGS "${_MKCONFIG_32BIT_FLAGS}"
   fi
 }
 
@@ -116,7 +115,7 @@ check_using_cplusplus () {
   esac
 
   printyesno_val _MKCONFIG_USING_CPLUSPLUS "${usingcplusplus}"
-  setdata ${_MKCONFIG_PREFIX} _MKCONFIG_USING_CPLUSPLUS "${usingcplusplus}"
+  setdata _MKCONFIG_USING_CPLUSPLUS "${usingcplusplus}"
 }
 
 check_using_gcc () {
@@ -139,7 +138,7 @@ check_using_gcc () {
   esac
 
   printyesno_val _MKCONFIG_USING_GCC "${usinggcc}"
-  setdata ${_MKCONFIG_PREFIX} _MKCONFIG_USING_GCC "${usinggcc}"
+  setdata _MKCONFIG_USING_GCC "${usinggcc}"
 }
 
 check_using_clang () {
@@ -162,7 +161,7 @@ check_using_clang () {
   esac
 
   printyesno_val _MKCONFIG_USING_CLANG "${usingclang}"
-  setdata ${_MKCONFIG_PREFIX} _MKCONFIG_USING_CLANG "${usingclang}"
+  setdata _MKCONFIG_USING_CLANG "${usingclang}"
 }
 
 check_using_gnu_ld () {
@@ -180,7 +179,7 @@ check_using_gnu_ld () {
   fi
 
   printyesno_val _MKCONFIG_USING_GNU_LD "${usinggnuld}"
-  setdata ${_MKCONFIG_PREFIX} _MKCONFIG_USING_GNU_LD "${usinggnuld}"
+  setdata _MKCONFIG_USING_GNU_LD "${usinggnuld}"
 }
 
 check_cflags () {
@@ -204,7 +203,8 @@ check_cflags () {
         fi
         ;;
     esac
-    doappend cflags_compiler " $gccflags"
+    doappend cflags_compiler " "
+    doappend cflags_compiler $gccflags
   fi
   if [ "${_MKCONFIG_USING_CLANG}" = Y ]; then
     puts "set clang flags" >&9
@@ -277,22 +277,21 @@ check_cflags () {
   _dogetconf
 
   # largefile flags
-  doappend cflags_system " $lfccflags"
+  doappend cflags_system " "
+  doappend cflags_system $lfccflags
 
   # plain CFLAGS will be interpreted as the user's cflags
-  _read_option CFLAGS ""
   cflags_user=$CFLAGS
-  _read_option CFLAGS_DEBUG ""
   if [ "z$CFLAGS_DEBUG" != z ]; then
     cflags_debug="$CFLAGS_DEBUG"
   fi
-  _read_option CFLAGS_OPTIMIZE ""
   if [ "z$CFLAGS_OPTIMIZE" != z ]; then
     cflags_optimize="$CFLAGS_OPTIMIZE"
   fi
-  _read_option CFLAGS_INCLUDE ""
-  doappend cflags_include " $CFLAGS_INCLUDE"
-  doappend cflags_include " $CPPFLAGS"
+  doappend cflags_include " "
+  doappend cflags_include $CFLAGS_INCLUDE
+  doappend cflags_include " "
+  doappend cflags_include $CPPFLAGS
 
   puts "cflags_debug:${cflags_debug}" >&9
   puts "cflags_optimize:${cflags_optimize}" >&9
@@ -314,15 +313,36 @@ check_cflags () {
 
 check_addcflag () {
   name=$1
-  flag=$2
+  shift
+  flag=$*
 
   printlabel CFLAGS_APPLICATION "Add C flag: ${flag}"
 
   test_cflag "$flag"
   printyesno $name "${flag}"
-  if [ $flag != 0 ]; then
-    doappend CFLAGS_APPLICATION " $flag"
-    setdata ${_MKCONFIG_PREFIX} CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
+  if [ "$flag" != 0 ]; then
+    doappend CFLAGS_APPLICATION " "
+    doappend CFLAGS_APPLICATION "$flag"
+    setdata CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
+  fi
+}
+
+check_addincpath () {
+  name=$1
+  flag=$2
+
+  printlabel CFLAGS_APPLICATION "Add Include Path: ${flag}"
+
+  test_incpath "$flag"
+  printyesno $name "${flag}"
+  if [ "$flag" != 0 ]; then
+    dosubst flag ' ' '\\\\\\\\\ '
+    doappend CFLAGS_APPLICATION " "
+    initifs
+    setifs
+    doappend CFLAGS_APPLICATION $flag
+    resetifs
+    setdata CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
   fi
 }
 
@@ -347,8 +367,10 @@ check_pkg_cflags () {
   test_cflag "$tcflags"
   printyesno_val $name "${flag}"
   if [ "$flag" != 0 ]; then
-    doappend CFLAGS_APPLICATION " $flag"
-    setdata ${_MKCONFIG_PREFIX} CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
+    doappend CFLAGS_APPLICATION " "
+    doappend CFLAGS_APPLICATION "$flag"
+    setdata CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
+    setdata ${name} "$flag"
   fi
 }
 
@@ -373,8 +395,10 @@ check_pkg_include () {
   test_cflag "$tcflags"
   printyesno_val $name "${flag}"
   if [ "$flag" != 0 ]; then
-    doappend CFLAGS_APPLICATION " $flag"
-    setdata ${_MKCONFIG_PREFIX} CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
+    doappend CFLAGS_APPLICATION " "
+    doappend CFLAGS_APPLICATION "$flag"
+    setdata CFLAGS_APPLICATION "$CFLAGS_APPLICATION"
+    setdata ${name} "$flag"
   fi
 }
 
@@ -400,21 +424,59 @@ check_pkg_libs () {
   test_ldflags "$tldflags"
   printyesno_val $name "$flag"
   if [ "$flag" != 0 ]; then
-    doappend LDFLAGS_LIBS_APPLICATION " $flag"
-    setdata ${_MKCONFIG_PREFIX} LDFLAGS_LIBS_APPLICATION "$LDFLAGS_LIBS_APPLICATION"
+    doappend LDFLAGS_LIBS_APPLICATION " "
+    doappend LDFLAGS_LIBS_APPLICATION "$flag"
+    setdata LDFLAGS_LIBS_APPLICATION "$LDFLAGS_LIBS_APPLICATION"
+    setdata ${name} "$flag"
   fi
 }
 
 check_addldflag () {
   name=$1
-  flag=$2
+  shift
+  flag=$*
 
   printlabel LDFLAGS_APPLICATION "Add LD flag: ${flag}"
 
   test_ldflags "$flag"
   printyesno $name "$flag"
-  if [ $flag != 0 ]; then
-    doappend ldflags_application " $flag"
+  if [ "$flag" != 0 ]; then
+    doappend ldflags_application " "
+    doappend ldflags_application "\"$flag\""
+    _setflags ldflags_application LDFLAGS_APPLICATION
+  fi
+}
+
+check_addlibs () {
+  name=$1
+  shift
+  flag=$*
+
+  printlabel LDFLAGS_LIBS_APPLICATION "Add Library: ${flag}"
+
+  test_ldflags "$flag"
+  printyesno $name "$flag"
+  if [ "$flag" != 0 ]; then
+    doappend ldflags_libs_application " "
+    doappend ldflags_libs_application "\"$flag\""
+    _setflags ldflags_libs_application LDFLAGS_LIBS_APPLICATION
+  fi
+}
+
+check_addldsrchpath () {
+  name=$1
+  flag=$2
+
+  printlabel LDFLAGS_APPLICATION "Add LD Search Path: ${flag}"
+
+  test_ldsrchpath "$flag"
+  printyesno $name "$flag"
+  if [ "$flag" != 0 ]; then
+    doappend ldflags_application " "
+    initifs
+    setifs
+    doappend ldflags_application "\"$flagi\""
+    resetifs
     _setflags ldflags_application LDFLAGS_APPLICATION
   fi
 }
@@ -432,7 +494,8 @@ check_ldflags () {
   ldflags_system=${LDFLAGS_SYSTEM:-}
   ldflags_application=${LDFLAGS_APPLICATION:-}
 
-  doappend ldflags_system " $lfldflags"
+  doappend ldflags_system " "
+  doappend ldflags_system $lfldflags
 
   case ${_MKCONFIG_SYSTYPE} in
       FreeBSD)
@@ -477,23 +540,18 @@ check_ldflags () {
   _dogetconf
 
   # plain LDFLAGS will be interpreted as the user's ldflags
-  _read_option LDFLAGS ""
   ldflags_user=$LDFLAGS
-  _read_option LDFLAGS_DEBUG ""
   if [ "z$LDFLAGS_DEBUG" != z ]; then
     ldflags_debug="$LDFLAGS_DEBUG"
   else
-    _read_option CFLAGS_DEBUG ""
     if [ "z$CFLAGS_DEBUG" != z ]; then
       ldflags_debug="$CFLAGS_DEBUG"
     fi
   fi
 
-  _read_option LDFLAGS_OPTIMIZE ""
   if [ "z$LDFLAGS_OPTIMIZE" != z ]; then
     ldflags_optimize="$LDFLAGS_OPTIMIZE"
   else
-    _read_option CFLAGS_OPTIMIZE ""
     if [ "z$CFLAGS_OPTIMIZE" != z ]; then
       ldflags_optimize="$CFLAGS_OPTIMIZE"
     fi
@@ -535,10 +593,11 @@ check_libs () {
   esac
 
   # largefile flags
-  doappend ldflags_libs_system " $lflibs"
+  doappend ldflags_libs_system " "
+  doappend ldflags_libs_system $lflibs
 
-  _read_option LIBS ""
-  doappend ldflags_libs_user " $LIBS"
+  doappend ldflags_libs_user " "
+  doappend ldflags_libs_user $LIBS
 
   puts "ldflags_libs_user:${ldflags_libs_user}" >&9
   puts "ldflags_libs_application:${ldflags_libs_application}" >&9
@@ -560,7 +619,7 @@ check_cflags_shared () {
 
   cflags_shared=""
 
-  if [ "$_MKCONFIG_USING_GCC" != Y ]; then
+  if [ "$_MKCONFIG_USING_GCC" != Y -a "$_MKCONFIG_USING_CLANG" != Y ]; then
     case ${_MKCONFIG_SYSTYPE} in
       CYGWIN*|MSYS*|MINGW*)
         # apparently, clang does not need this any more.
@@ -596,7 +655,6 @@ check_cflags_shared () {
     doappend cflags_shared " -fPIC"
   fi
 
-  _read_option CFLAGS_SHARED ""
   if [ "z$CFLAGS_SHARED" != z ]; then
     cflags_shared_user="${CFLAGS_SHARED}"
   fi
@@ -615,7 +673,7 @@ check_shldflags () {
 check_ldflags_shared () {
   printlabel LDFLAGS_SHARED_LIBLINK "shared library ldflags"
 
-  if [ "$_MKCONFIG_USING_GCC" != Y ]; then
+  if [ "$_MKCONFIG_USING_GCC" != Y -a "$_MKCONFIG_USING_CLANG" != Y ]; then
     case ${_MKCONFIG_SYSTYPE} in
       AIX)
         doappend ldflags_shared_liblink " -G"
@@ -650,7 +708,6 @@ check_ldflags_shared () {
     doappend ldflags_shared_liblink " -shared"
   fi
 
-  _read_option LDFLAGS_SHARED ""
   if [ "z$LDFLAGS_SHARED" != z ]; then
     ldflags_shared_user="${LDFLAGS_SHARED}"
   fi
@@ -688,7 +745,71 @@ check_sharednameflag () {
   fi
 
   printyesno_val SHLDNAMEFLAG "$SHLDNAMEFLAG"
-  setdata ${_MKCONFIG_PREFIX} SHLDNAMEFLAG "$SHLDNAMEFLAG"
+  setdata SHLDNAMEFLAG "$SHLDNAMEFLAG"
+}
+
+check_sharedliblinkflag () {
+  printlabel LDFLAGS_SHARED_LIB_LINK "link flag for shared libraries "
+
+  LDFLAGS_SHARED_LIB_LINK="-Wl,-Bdynamic"
+  if [ "$_MKCONFIG_USING_GNU_LD" != Y ]; then
+    case ${_MKCONFIG_SYSTYPE} in
+      AIX)
+        LDFLAGS_SHARED_LIB_LINK=""
+        ;;
+      HP-UX)
+        LDFLAGS_SHARED_LIB_LINK=""
+        ;;
+      OSF1)
+        LDFLAGS_SHARED_LIB_LINK=""
+        ;;
+      SunOS)
+        # -Bdynamic
+        ;;
+      *)
+        LDFLAGS_SHARED_LIB_LINK=""
+        ;;
+    esac
+    if [ "$_MKCONFIG_USING_GCC" = Y -o "$_MKCONFIG_USING_CLANG" = Y ]; then
+      LDFLAGS_SHARED_LIB_LINK=`echo "$LDFLAGS_SHARED_LIB_LINK" |
+          sed -e 's/^-/-Wl,-/' -e 's/^\+/-Wl,+/' -e 's/  */ -Wl,/g'`
+    fi
+  fi
+
+  printyesno_val LDFLAGS_SHARED_LIB_LINK "$LDFLAGS_SHARED_LIB_LINK"
+  setdata LDFLAGS_SHARED_LIB_LINK "$LDFLAGS_SHARED_LIB_LINK"
+}
+
+check_staticliblinkflag () {
+  printlabel LDFLAGS_STATIC_LIB_LINK "link flag for static libraries "
+
+  LDFLAGS_STATIC_LIB_LINK="-Wl,-Bstatic"
+  if [ "$_MKCONFIG_USING_GNU_LD" != Y ]; then
+    case ${_MKCONFIG_SYSTYPE} in
+      AIX)
+        LDFLAGS_STATIC_LIB_LINK=""
+        ;;
+      HP-UX)
+        LDFLAGS_STATIC_LIB_LINK=""
+        ;;
+      OSF1)
+        LDFLAGS_STATIC_LIB_LINK=""
+        ;;
+      SunOS)
+        # -Bdynamic
+        ;;
+      *)
+        LDFLAGS_STATIC_LIB_LINK=""
+        ;;
+    esac
+    if [ "$_MKCONFIG_USING_GCC" = Y -o "$_MKCONFIG_USING_CLANG" = Y ]; then
+      LDFLAGS_STATIC_LIB_LINK=`echo "$LDFLAGS_STATIC_LIB_LINK" |
+          sed -e 's/^-/-Wl,-/' -e 's/^\+/-Wl,+/' -e 's/  */ -Wl,/g'`
+    fi
+  fi
+
+  printyesno_val LDFLAGS_STATIC_LIB_LINK "$LDFLAGS_STATIC_LIB_LINK"
+  setdata LDFLAGS_STATIC_LIB_LINK "$LDFLAGS_STATIC_LIB_LINK"
 }
 
 check_shareexeclinkflag () {
@@ -713,14 +834,14 @@ check_shareexeclinkflag () {
         LDFLAGS_EXEC_LINK=""
         ;;
     esac
-    if [ "$_MKCONFIG_USING_GCC" = Y ]; then
+    if [ "$_MKCONFIG_USING_GCC" = Y -o "$_MKCONFIG_USING_CLANG" = Y ]; then
       LDFLAGS_EXEC_LINK=`echo "$LDFLAGS_EXEC_LINK" |
           sed -e 's/^-/-Wl,-/' -e 's/^\+/-Wl,+/' -e 's/  */ -Wl,/g'`
     fi
   fi
 
   printyesno_val LDFLAGS_EXEC_LINK "$LDFLAGS_EXEC_LINK"
-  setdata ${_MKCONFIG_PREFIX} LDFLAGS_EXEC_LINK "$LDFLAGS_EXEC_LINK"
+  setdata LDFLAGS_EXEC_LINK "$LDFLAGS_EXEC_LINK"
 }
 
 check_sharerunpathflag () {
@@ -751,7 +872,7 @@ check_sharerunpathflag () {
         LDFLAGS_RUNPATH=""
         ;;
     esac
-    if [ "$_MKCONFIG_USING_GCC" = Y ]; then
+    if [ "$_MKCONFIG_USING_GCC" = Y -o "$_MKCONFIG_USING_CLANG" = Y ]; then
       # the trailing space will be converted to ' -Wl,' and
       # the library runpath will be appended by mkcl.sh
       LDFLAGS_RUNPATH=`echo "$LDFLAGS_RUNPATH" |
@@ -760,7 +881,7 @@ check_sharerunpathflag () {
   fi
 
   printyesno_val LDFLAGS_RUNPATH "$LDFLAGS_RUNPATH"
-  setdata ${_MKCONFIG_PREFIX} LDFLAGS_RUNPATH "$LDFLAGS_RUNPATH"
+  setdata LDFLAGS_RUNPATH "$LDFLAGS_RUNPATH"
 }
 
 check_addconfig () {
@@ -771,7 +892,8 @@ check_addconfig () {
   eval _tvar="\$$evar"
   if [ "z$_tvar" != z ]; then
     printyesno_val $name yes
-    doappend $addto " $_tvar"
+    doappend $addto " "
+    doappend $addto $_tvar
     puts "got: ${evar} ${_tvar}" >&9
     eval puts "\"$addto: \$$addto\"" >&9
     ucaddto=$addto
@@ -807,49 +929,12 @@ check_findconfig () {
 
   if [ z$sp != z ]; then
     printyesno_val $name yes
-    setdata ${_MKCONFIG_PREFIX} config_${cfile} Y
-    setdata ${_MKCONFIG_PREFIX} config_path_${cfile} $sp/$cfile
+    setdata config_${cfile} Y
+    setdata config_path_${cfile} $sp/$cfile
     . $sp/$cfile.sh ; # load the environment variables
   else
     printyesno_val $name no
-    setdata ${_MKCONFIG_PREFIX} config_${cfile} N
-  fi
-}
-
-check_findpc () {
-  name=$1
-  cfile=$2
-  printlabel FINDPC "Search for: ${cfile}"
-  sp=
-  incchk=
-  pp=`puts $PATH | sed 's/:/ /g'`
-  for p in $pp $HOME/local/lib /usr/local/lib \
-      /opt/local/lib /usr/lib/x86_64-linux-gnu; do
-    td=$p
-    case $p in
-      */bin)
-        dosubst td '/bin$' '/lib'
-        ;;
-    esac
-
-    doappend td /pkgconfig
-    if [ -d $td ]; then
-      if [ -f "$td/$cfile.sh" ]; then
-        puts "found: ${td}" >&9
-        sp=$td
-        break
-      fi
-    fi
-  done
-
-  if [ z$sp != z ]; then
-    printyesno_val $name yes
-    setdata ${_MKCONFIG_PREFIX} pc_${cfile} Y
-    setdata ${_MKCONFIG_PREFIX} pc_path_${cfile} $sp/$cfile
-    . $sp/$cfile.sh ; # load the environment variables
-  else
-    printyesno_val $name no
-    setdata ${_MKCONFIG_PREFIX} pc_${cfile} N
+    setdata config_${cfile} N
   fi
 }
 
@@ -862,3 +947,13 @@ check_standard_cc () {
   check_cflags
   check_ldflags
 }
+
+check_shared_flags () {
+  check_cflags_shared
+  check_ldflags_shared
+  check_sharednameflag
+  check_shareexeclinkflag
+  check_sharerunpathflag
+  check_sharedliblinkflag
+  check_staticliblinkflag
+}
